package fitconnect.api;

import fitconnect.api.auth.OAuthToken;
import fitconnect.api.data.Data;
import fitconnect.api.data.Metadata;
import fitconnect.api.validation.ValidationResult;

import java.io.ByteArrayInputStream;

/**
 * A technical system that creates a submission via the Submission API.
 */
public interface Sender {

    /**
     * Authenticates the sender against the FIT-Co API and retrieves an authentication token
     *
     * @param clientId - a unique identifier within the FIT-Co platform environment
     * @param clientSecret - a secret that is only known by the application and the OAuth server
     * @param scope - OAuth scope that determines if a submission is accepted by the client
     * @return {@link OAuthToken} that holds the access-token
     */
    OAuthToken retrieveAuthenticationToken(final String clientId, final String clientSecret, String scope);

    // TODO
    ValidationResult validateCertificateChain(byte[] chain);

    /**
     * Encrypts the submission data with JWE.
     *
     * @param unencryptedData - unencrypted JSON or XML data
     * @return encrypted JSON or XML data
     */
    Data encryptData(final Data unencryptedData);

    /**
     * Encrypts the submission attachment with JWE.
     *
     * @param unencryptedAttachmentBinary - the unencrypted binary data of the attachment
     * @return the encrypted binary of the attachment
     */
    ByteArrayInputStream encryptAttachment(final ByteArrayInputStream unencryptedAttachmentBinary);

    /**
     * Generates the metadata consisting of the attachment and its hash values.
     *
     * @param attachment - the unencrypted binary data of the attachment
     * @return metadata object
     */
    Metadata generateMetadata(byte[] attachment);

}
