package fitconnect.api;

import fitconnect.api.auth.OAuthToken;
import fitconnect.api.data.Data;
import fitconnect.api.data.Metadata;

import java.io.ByteArrayInputStream;

/**
 * The technical system that accepts submissions on the administration side.
 */
public interface Subscriber {

    /**
     * Authenticates the sender against the FitCo API and retrieves an authentication token
     * @param clientId -
     * @param clientSecret -
     * @return
     */
    OAuthToken retrieveAuthenticationToken(final String clientId, final String clientSecret);

    /**
     * Decrypts the subscribed data with JWE.
     *
     * @param encryptedData - encrypted JSON or XML data
     * @return decrypted JSON or XML data
     */
    Data decryptData(final Data encryptedData);

    /**
     * Encrypts the submission attachment with JWE.
     *
     * @param unencryptedAttachmentBinary - the unencrypted binary data of the attachment
     * @return the encrypted binary of the attachment
     */
    ByteArrayInputStream decryptAttachment(final ByteArrayInputStream unencryptedAttachmentBinary);

    /**
     * Generates the metadata consisting of the attachment and its hash values.
     *
     * @param attachment - the unencrypted binary data of the attachment
     * @return metadata object
     */
    Metadata generateMetadata(byte[] attachment);


}
